.hxr-hoverxref {
    border-bottom: 1px dotted;
    border-color: gray;
}

.tooltipster-box {
    max-height: 600px;
}

.tooltipster-sidetip.tooltipster-shadow.tooltipster-shadow-custom .tooltipster-content {
    color: #8d8d8d;
    font-size: 80%;
    padding: 18px;
}

.tooltipster-sidetip.tooltipster-shadow.tooltipster-shadow-custom .tooltipster-content p {
    font-size: 100% !important;
}

.tooltipster-sidetip.tooltipster-shadow.tooltipster-shadow-custom .tooltipster-content .headerlink {
    visibility: hidden;
}